#include "mainwindow.h"
#include "dbio.h"
#include "brlcad/ConstDatabase.h"
#include "brlcad/Object.h"
#include "brlcad/Combination.h"
#include <QApplication>
#include <QMenuBar>
#include <QAction>
#include <QKeySequence>
#include <QFileDialog>
#include <QStringList>
#include <QListWidget>
#include <QVBoxLayout>
#include <QMessageBox>
#include <string>
#include <iostream>


void
MainWindow::setDatabaseName(const char *databaseName)
{
	std::string title = "";
	if (databaseName) {
		title += databaseName;
		title += " - BRLCAD-GUI";
	} else {
		title += "BRLCAD-GUI";
	}
	setWindowTitle(title.c_str());
}


void
MainWindow::openFileDialog()
{
	QFileDialog *dialog = new QFileDialog();
	QStringList *filters = new QStringList();
	*filters << "BRL-CAD Database Files (*.g)"
		    << "All files (*)";
	dialog->setNameFilters(*filters);
	dialog->setFileMode(QFileDialog::AnyFile);
	if (dialog->exec()) {
    	QStringList fileNames = dialog->selectedFiles();
    	std::string fileName = fileNames[0].toStdString();
    	const char *fileNamePtr = fileName.c_str();
    	std::cout << fileNamePtr << std::endl;
    	if (database)
    	 	closeDatabase();
    	loadDatabase(fileNamePtr);
	}
}


void
MainWindow::loadDatabase(const char *fileName)
{
	if (DatabaseUtils::openDatabase(database, fileName) == 0) {
		setDatabaseName(database->Title());
		std::vector<const char*> objects = loadTopLevelObjects();
		setInterfaceVisible(true);
    }
}

void
MainWindow::closeDatabase()
{
	setDatabaseName(NULL);
	setInterfaceVisible(false);
}


void
MainWindow::setInterfaceVisible(bool visible)
{
	if (visible) {
		actionClose->setVisible(true);

	} else {
		actionClose->setVisible(false);
		objectListWidget->clear();
	}
}


void
MainWindow::quit()
{
	QApplication::instance()->quit();
}


void
MainWindow::initMenu()
{
	QWidget *central = new QWidget(this);
	QMenuBar *menuBar = new QMenuBar(0);

	actionOpen = new QAction(tr("&Open Database"), this);
	actionOpen->setShortcut(QKeySequence::Open);
	connect(actionOpen, &QAction::triggered, this, &MainWindow::openFileDialog);


	actionClose = new QAction(tr("&Close Database"), this);
	actionClose->setShortcut(QKeySequence::Close);
	connect(actionClose, &QAction::triggered, this, &MainWindow::closeDatabase);


	actionQuit = new QAction(tr("&Quit"), this);
	actionQuit->setShortcut(QKeySequence::Quit);
	connect(actionQuit, &QAction::triggered, this, &MainWindow::quit);


	QMenu *fileMenu = menuBar->addMenu(tr("&File"));
	fileMenu->addAction(actionOpen);
	fileMenu->addSeparator();
	fileMenu->addAction(actionClose);
	fileMenu->addSeparator();
	fileMenu->addAction(actionQuit);

	objectListWidget = new QListWidget(this);
	connect(objectListWidget, &QListWidget::itemClicked, 
            this, &MainWindow::onObjectClicked);

	QVBoxLayout *verticalLayout = new QVBoxLayout(this);
	verticalLayout->addWidget(menuBar);
	verticalLayout->addWidget(objectListWidget);

	menuBar->show();

	central->setLayout(verticalLayout);
	setCentralWidget(central);
	std::cout << "wew1" <<std::endl;
}


void
MainWindow::onObjectClicked(QListWidgetItem* listItem)
{
	std::string objectName = listItem->text().toStdString();
	BRLCAD::Vector3D boxMinima = database->BoundingBoxMinima();
	BRLCAD::Vector3D boxMaxima = database->BoundingBoxMaxima();
	const char *objectNamePtr = objectName.c_str();
	const double *minimaCoordinates = boxMinima.coordinates;
	const double *maximaCoordinates = boxMaxima.coordinates;
	QMessageBox *messageBox = new QMessageBox(this);
	QString windowTitle = QString("Object \"%1\"").arg(objectNamePtr);

	messageBox->setWindowTitle(windowTitle);
	QString minimaText = "BoundingBoxMinima: (%1, %2, %3)";
	minimaText = minimaText.arg(minimaCoordinates[0]).arg(minimaCoordinates[1]).arg(minimaCoordinates[2]);
	QString maximaText = "BoundingBoxMaxima: (%1, %2, %3)";
	maximaText = maximaText.arg(maximaCoordinates[0]).arg(maximaCoordinates[1]).arg(maximaCoordinates[2]);
	messageBox->setText(minimaText+"\n"+maximaText);
	messageBox->exec();
}


std::vector<const char*>
MainWindow::loadTopLevelObjects()
{
	std::vector<const char*> objects;

	BRLCAD::ConstDatabase::TopObjectIterator iter = database->FirstTopObject();
	while (iter.Good())  {
		const char *objectName = iter.Name();

		std::cout << objectName << std::endl;
		objects.push_back(objectName);
		++iter;
	}
	for (int i = 0; i < objects.size(); i++)
		if (strcmp(objects[i],"_GLOBAL") != 0)
			objectListWidget->addItem(QString(objects[i]));
	return objects;
} 


MainWindow::MainWindow(QWidget *parent): QMainWindow(parent)
{
	database = new BRLCAD::ConstDatabase();
	initMenu();
	setMinimumSize(640, 400);
}


MainWindow::~MainWindow()
{
	
}
